/*
Sand Art Display-2 is a revised program for an X-Y sand table drawing multiple patterns with two stepper motors.
Patterns are selected with a Liquid Cristal Display and UP/DOWN buttons.
The pushbuttons are connected to resistors in series connected on analog input A1, allowing different values to be entered.   
This program drives two unipolar 5V DC stepper motors type 28BYJ-48.
These motors are attached to digital pins 4,5,6,7 and 8,9,10 and 11 of the Arduino UNO, via a driver board with IC ULN2003.
The motor power supply is a separate 5V DC supply. 
The motors drive both axis of the sand art table with a 250 mm rack gear.
Start position: The motors are resting in the full left & up position.
See www.apinventions.wordpress.com for the Sand Art Display-2 building instructions.

Created February 2022 by Alex Pikkert.
*/

//this sequence is for the 28BYJ-48 stepper motor.
#include <Stepper.h>
Stepper myStepperX(1024, 8,10,9,11);Stepper myStepperY(1024, 4,6,5,7);

//this sequence is for the LCD 16x2 connected via I2C.
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27,16,2); 

//**************************************************************************
// variables used in the motor control and in the pattern subroutines.
int button    = A1; // input pin for analog button value 390(up)/621(down)/810(go)
int select    = 0;  // select which road to follow... 
int xstop     = 12; // input pin end position left or right
int ystop     = 3;  // input pin end position up or down
int speed1    = 10; // slow speed
int speed2    = 15; // medium speed 
int speed3    = 25; // fast speed
int speed4    = 30; // super fast speed
int up        = 50; // length of standard step up
int down      = -50;// length of standard step down  
int left      = 50; // length of standard step left
int right     = -50;// length of standard step right
int n=0;            // counter flag up/down
int m=0;            // counter flag left/right
int x=0;            // left/right number of steps
int y=0;            // up/down number of steps
int d=0;            // counter for step difference
long randNumber;    // counter for random steps
double cosinus;     // cosine value for X
double sinus;       // sine value for Y
int i;              // steps counter
int j;              // steps counter
int ystep;          // Y step increment
int yprev;          // Y step store in memory 
int ymax;           // Y max amplitude 
int xmax;           // X max amplitude 
int xstep;          // X step increment
int xprev;          // X step store in memory 
int radius;         // circle diameter
int a;              // select pattern on display
int b;              // select button value on A1
int go;             // start angle of a circle
int finish;         // end angle of a circle

// arrow signs up/down for display
byte arrowdown[8] = {
  B00100,
  B00100,
  B00100,
  B00100,
  B11111,
  B01110,
  B00100,
};

byte arrowup[8] = {
  B00100,
  B01110,
  B11111,
  B00100,
  B00100,
  B00100,
  B00100,
  
};

// Interrupt Service Routine (ISR) to stop running pattern
void buttonPressed ()
{
  if (digitalRead (2) == LOW)
    asm volatile ("  jmp 0"); 
}  

//**************************************************************************
void setup() {
// Initialize the display
lcd.init();
  lcd.clear();
  lcd.backlight(); 
  lcd.setCursor(0,0);
  lcd.print("SAND ART DISPLAY");
  lcd.setCursor(0,1);
  lcd.print("**  version 2 **");
delay(3000);

//initial speed setting for both motors
myStepperX.setSpeed(speed3);myStepperY.setSpeed(speed3);

pinMode(3,INPUT_PULLUP);  // contact input to detect UP/DOWN end position
pinMode(12,INPUT_PULLUP); // contact input to detect LEFT/RIGHT end position
pinMode(A1,INPUT_PULLUP); // analog input value for the pushbuttons
pinMode (2,INPUT_PULLUP); // button input to stop running pattern  
attachInterrupt (digitalPinToInterrupt (2), buttonPressed, LOW);  // attach interrupt handler

startposition();
delay(2000);
lcd.setCursor(0,0);lcd.print("SELECT     & GO!");
lcd.setCursor(0,1);lcd.print("WAFFLE          ");

lcd.createChar(0, arrowup);
lcd.setCursor(7,0);
lcd.write(byte(0));

lcd.createChar(1, arrowdown);
lcd.setCursor(9,0);
lcd.write(byte(1));

a=1;
}

//**************************************************************************
void loop() {
randomSeed(analogRead(A0));

b= analogRead(button);
if (b >250 && b <700) {Selectpattern();}

// avoid erroneous trigger of GO subroutine 
if (b>800 && b <820) {
  delay(500);
  b= analogRead(button);
  if(b>800 && b<820){
    lcd.setCursor(0,0);
    lcd.print("Running...      ");
    Go();
  }
  }
}

//*************************** SUBROUTINES **********************************

void startposition()
{
// Move to start position
lcd.setCursor(0,0);lcd.print("Wait...         ");
lcd.setCursor(0,1);lcd.print("Moving to start ");
myStepperX.setSpeed(speed4);myStepperY.setSpeed(speed4);
myStepperY.step(8*up);myStepperX.step(8*left);
while (digitalRead(ystop)==HIGH){myStepperY.step(4*up);}
while (digitalRead(xstop)==HIGH){myStepperX.step(4*left);}
}

//**************************************************************************

void Selectpattern()
{ delay(500);
  //scroll down in the list of patterns
    if (b >600 && b <700) {a = a + 1;}
  //scroll up in the list of patterns
    if (b >250 && b <450) {a = a - 1;}

//this is the list of patterns, new patterns can be added 
  if (a < 1  ) {a = 9;}
  if (a == 10 ) {a = 1;}

  if (a == 1) {lcd.setCursor(0, 1);lcd.print("WAFFLE          ");return;}
  if (a == 2) {lcd.setCursor(0, 1);lcd.print("PLOUGH          ");return;}
  if (a == 3) {lcd.setCursor(0, 1);lcd.print("PONG            ");return;}
  if (a == 4) {lcd.setCursor(0, 1);lcd.print("SINE            ");return;}
  if (a == 5) {lcd.setCursor(0, 1);lcd.print("HELLO WORLD     ");return;}
  if (a == 6) {lcd.setCursor(0, 1);lcd.print("MAD ANT         ");return;}
  if (a == 7) {lcd.setCursor(0, 1);lcd.print("SPIRAL          ");return;}
  if (a == 8) {lcd.setCursor(0, 1);lcd.print("CHRISTMAS TREE  ");return;}
  if (a == 9) {lcd.setCursor(0, 1);lcd.print("FERRARI         ");return;}
}

//**************************************************************************

void Go()
{ 
 delay(500); 
if (a==1) {waffle();}  
if (a==2) {plough();}  
if (a==3) {pong();}  
if (a==4) {sine();}  
if (a==5) {helloworld();}  
if (a==6) {madant();}  
if (a==7) {spiral();}  
if (a==8) {christmastree();}  
if (a==9) {ferrari();}  
}

//**************************************************************************

void drawsinus(){
for (i = 1; i <= 360; i=i+2)
 {sinus = sin(i*0.0175);
 y=(sinus*ymax);
 ystep=(y-yprev);
 myStepperY.step(ystep);
 myStepperX.step(0.44*right);
 yprev=y;}
 
 myStepperX.step(76*left);
}

//**************************************************************************
void drawspiral(){

for (i = 90; i <450; i=i+2)
{
ymax=radius;xmax=radius;
sinus = sin(i*0.0175);
cosinus = cos(i*0.0175);
y=(sinus*ymax);
x=(cosinus*xmax);
ystep=(y-yprev); yprev=y;
xstep=(xprev-x); xprev=x;
myStepperY.step(ystep);
myStepperX.step(-xstep);
radius =radius-1;
}
}

//**************************************************************************
void drawcircle(){
 for (i = go; i <finish; i=i+2)
{
ymax=radius;xmax=radius;
sinus = sin(i*0.0175);
cosinus = cos(i*0.0175);
y=(sinus*ymax);
x=(cosinus*xmax);
ystep=(y-yprev); yprev=y;
xstep=(xprev-x); xprev=x;
myStepperY.step(ystep);
myStepperX.step(-xstep);
}
}

//*************************** SUBROUTINES FOR PATTERNS**********************
//WAFFLE 
void waffle(){
myStepperX.setSpeed(speed3);myStepperY.setSpeed(speed3);
if(d==0){
if(n==0){myStepperY.step(5*down);}
y = digitalRead(ystop);if(y==LOW && n==0){myStepperX.step(11*right);n=1;x=x+1;}
if(n==1){myStepperY.step(5*up);}
y = digitalRead(ystop);if(y==LOW && n==1){myStepperX.step(11*right);n=0;x=x+1;}
if(x >7){d=1;y=0;n=0;}
}
if (d==1){
if(n==0){myStepperX.step(5*left);}
x = digitalRead(xstop);if(x==LOW && n==0){myStepperY.step(11*down);n=1;y=y+1;}
if(n==1){myStepperX.step(5*right);}
x = digitalRead(xstop);if(x==LOW && n==1){myStepperY.step(11*down);n=0;y=y+1;}
while(y >5){lcd.setCursor(0,0);lcd.print("Ready-press STOP");}
}
waffle();
}

//**************************************************************************
//PLOUGH
void plough(){
myStepperX.setSpeed(speed3);myStepperY.setSpeed(speed3);
x=73-d;myStepperX.step(x*right);
y=56-d;myStepperY.step(y*down);
d=d+3;
x=73-d;myStepperX.step(x*left);
y=56-d;myStepperY.step(y*up);
d=d+3;
while(y<5 || x< 5){lcd.setCursor(0,0);lcd.print("Ready-press STOP");}
plough();
}

//**************************************************************************
//PONG
void pong(){
lcd.setCursor(10, 1);lcd.print("*STOP*");
myStepperX.setSpeed(speed2);myStepperY.setSpeed(speed2);
y = digitalRead(ystop);if(y==LOW && n==0){myStepperY.step(5*down);n=1;}
y = digitalRead(ystop);if(y==HIGH && n==1){myStepperY.step(2*down);}
y = digitalRead(ystop);if(y==LOW && n==1){myStepperY.step(5*up);n=0;} 
y = digitalRead(ystop);if(y==HIGH && n==0){myStepperY.step(2*up);}

x = digitalRead(xstop);if(x==LOW && m==0){myStepperX.step(5*right);m=1;}
x = digitalRead(xstop);if(x==HIGH && m==1){myStepperX.step(2*right);}
x = digitalRead(xstop);if(x==LOW && m==1){myStepperX.step(5*left);m=0;} 
x = digitalRead(xstop);if(x==HIGH && m==0){myStepperX.step(2*left);}
pong();
}

//**************************************************************************
//SINE 
void sine(){
myStepperX.setSpeed(speed4);myStepperY.setSpeed(speed4);
if (d==0)
{myStepperY.step(60*down);myStepperY.step(30*up);
myStepperX.step(76*right);myStepperX.step(76*left);
d=1;delay(500);}
ymax=1400;drawsinus();
ymax=900;drawsinus();
ymax=400;drawsinus();
ymax=-1400;drawsinus();
ymax=-900;drawsinus();
ymax=-400;drawsinus();
while(i>360){lcd.setCursor(0,0);lcd.print("Ready-press STOP");}
}

//**************************************************************************
//MESSAGE HELLO WORLD
void helloworld(){
x=9;y=12;
myStepperX.setSpeed(speed4);myStepperY.setSpeed(speed4);
//H
myStepperY.step(2*y*down);
myStepperY.step(y*up);
myStepperX.step(x*right);
myStepperY.step(y*up);
myStepperY.step(2*y*down);

//E
myStepperX.step(x*right);
myStepperY.step(y*up);
myStepperX.step(x*right);
myStepperX.step(x*left);
myStepperY.step(y*down/2);
myStepperX.step(x*right);
myStepperX.step(x*left);
myStepperY.step(y*down/2);
myStepperX.step(2*x*right);

//LL
myStepperY.step(2*y*up);
myStepperY.step(2*y*down);
myStepperX.step(x*right);
myStepperY.step(2*y*up);
myStepperY.step(2*y*down);

//O
myStepperX.step(x*right);
myStepperY.step(y*up);
myStepperX.step(x*right);
myStepperY.step(y*down);
myStepperX.step(1.3*x*left);
myStepperX.step(2.3*x*right);
myStepperY.step(2.8*y*down);
myStepperX.step(8*x*left);

//W
myStepperY.step(y*up);
myStepperY.step(y*down);
myStepperX.step(0.6*x*right);
myStepperY.step(0.5*y*up);
myStepperY.step(0.5*y*down);
myStepperX.step(0.6*x*right);
myStepperY.step(y*up);
myStepperY.step(y*down);

//O
myStepperX.step(x*right);
myStepperY.step(y*up);
myStepperX.step(x*right);
myStepperY.step(y*down);
myStepperX.step(1.3*x*left);
myStepperX.step(2.3*x*right);

//R
myStepperY.step(y*up);
myStepperX.step(x*right);
myStepperX.step(x*left);
myStepperY.step(y*down);

//L
myStepperX.step(1.8*x*right);
myStepperY.step(2*y*up);
myStepperY.step(2*y*down);

//D
myStepperX.step(x*right);
myStepperY.step(y*up);
myStepperX.step(x*right);
myStepperY.step(y*down);
myStepperX.step(0.3*x*right);
myStepperX.step(0.3*x*left);
while(x==9){lcd.setCursor(0,0);lcd.print("Ready-press STOP");}
}

//**************************************************************************
//MAD ANT
void madant(){
lcd.setCursor(10, 1);lcd.print("*STOP*");  
myStepperX.setSpeed(speed4);myStepperY.setSpeed(speed4);
if (d==0)
{myStepperX.step(10*right);myStepperY.step(10*down);}
d=1;
randNumber= random(-10,10);myStepperX.step(randNumber*left);
x= digitalRead(xstop);if(x==LOW) 
{myStepperX.step(5*left);
x= digitalRead(xstop);if(x==HIGH) {myStepperX.step(15*left);}
else{myStepperX.step(15*right);}
}
randNumber= random(-10,10);myStepperY.step(randNumber*down);
y= digitalRead(ystop);if(y==LOW) 
{myStepperY.step(5*down);
y= digitalRead(ystop);if(y==HIGH) {myStepperY.step(15*down);}
else{myStepperY.step(15*up);}
}
madant();
}

//**************************************************************************
//SPIRAL
void spiral(){
myStepperX.setSpeed(speed3);myStepperY.setSpeed(speed3);
if (d==0)
{myStepperX.step(36*right);
radius=1500;ymax=radius; xmax=radius;yprev=radius;
d=1;delay(500);}
for (j = 1; j<9; j=j+1)
{drawspiral();}
while(d==1){lcd.setCursor(0,0);lcd.print("Ready-press STOP");}
}

//**************************************************************************
//CHRISTMAS TREE
void christmastree(){
myStepperX.setSpeed(speed4);myStepperY.setSpeed(speed4);
y=56;myStepperY.step(y*down);
x=74;myStepperX.step(x*right);
x=36;myStepperX.step(x*left);
y=10;myStepperY.step(y*up);
for (i = 1; i <7; i=i+1)
{
j=5;
x=37-i*j;myStepperX.step(x*left);
x=2*(37-i*j);myStepperX.step(x*right);
x=37-i*j;myStepperX.step(x*left);
y=6;myStepperY.step(y*up);
}
y=5;myStepperY.step(y*up);
while(y>0){lcd.setCursor(0,0);lcd.print("Ready-press STOP");}
}

//**************************************************************************
//FERRARI
void ferrari(){
myStepperX.setSpeed(speed4);myStepperY.setSpeed(speed4);
y=45;myStepperY.step(y*down);
x=76;myStepperX.step(x*right);
x=52;myStepperX.step(x*left);
radius=300; go=270; finish=720; drawcircle();
x=10;myStepperX.step(x*left);
y=15;myStepperY.step(y*up);
x=60;myStepperX.step(x*right);
x=50;myStepperX.step(x*left);
for (j = 1; j <6; j=j+1)
{
x=j;myStepperX.step(x*right);
y=j;myStepperY.step(y*up);
}
x=10;myStepperX.step(x*right);
y=15;myStepperY.step(y*down);
y=15;myStepperY.step(y*up);
x=20;myStepperX.step(x*right);
y=15;myStepperY.step(y*down);
x=10;myStepperX.step(x*right);
y=15;myStepperY.step(y*down);
x=25;myStepperX.step(x*left);
radius=300; go=180; finish=720; drawcircle();
x=10;myStepperX.step(x*left);
while(radius>0){lcd.setCursor(0,0);lcd.print("Ready-press STOP");}
}
